#!/usr/bin/python3

"""Nexia Climate Device Access"""

import argparse
import code
import readline
import rlcompleter

from nexia.home import NexiaHome

parser = argparse.ArgumentParser()
parser.add_argument("--username", type=str, help="Your Nexia username/email address.")
parser.add_argument("--password", type=str, help="Your Nexia password.")

args = parser.parse_args()

if args.username and args.password:
    nexia_home = NexiaHome(username=args.username, password=args.password)
else:
    parser.print_help()
    exit()

print("NexiaThermostat instance can be referenced using nt.<command>.")
print("List of available thermostats and zones:")
for _thermostat_id in nexia_home.get_thermostat_ids():
    thermostat = nexia_home.get_thermostat_by_id(_thermostat_id)
    _thermostat_name = thermostat.get_name()
    _thermostat_model = thermostat.get_model()
    _thermostat_compressor_speed = thermostat.get_current_compressor_speed()

    print(
        f'{_thermostat_id} - "{_thermostat_name}" ({_thermostat_model}) [{_thermostat_compressor_speed}]'
    )
    print("  Zones:")

    for _zone_id in thermostat.get_zone_ids():
        zone = thermostat.get_zone_by_id(_zone_id)
        _zone_name = zone.get_name()
        _zone_status = zone.get_status()

        print(f'    {_zone_id} - "{_zone_name}" ({_zone_status})')
del (
    _thermostat_id,
    _thermostat_model,
    _thermostat_name,
    _zone_name,
    _zone_id,
    args,
    parser,
)

nexia_home.update()


variables = globals()
variables.update(locals())

readline.set_completer(rlcompleter.Completer(variables).complete)
readline.parse_and_bind("tab: complete")
code.InteractiveConsole(variables).interact()



"""Nexia Automation."""

import logging

_LOGGER = logging.getLogger(__name__)


class NexiaAutomation:
    """A nexia Automation.

    Represents a nexia automation.
    """

    def __init__(self, nexia_home, automation_json):
        """Init nexia Thermostat."""
        self._nexia_home = nexia_home
        self.automation_id = automation_json["id"]
        self._automation_json = automation_json

    @property
    def API_MOBILE_AUTOMATION_URL(self):  # pylint: disable=invalid-name
        return self._nexia_home.mobile_url + "/automations/{automation_id}/{end_point}"

    @property
    def name(self):
        """
        Name of the automation.
        """
        return self._get_automation_key("name")

    @property
    def description(self):
        """
        Description of the automation.
        """
        return self._get_automation_key("description")

    @property
    def enabled(self):
        """
        Enabled stat of the automation.
        """
        return self._get_automation_key("enabled")

    def activate(self):
        """
        run the automation.
        """
        return self._post_automation_json("activate", "")

    def _get_automation_key(self, key):
        """
        Returns the automation value from the provided key in the automation's
        JSON.
        :param key: str
        :return: value
        """
        automation = self._automation_json
        if key in automation:
            return automation[key]
        raise KeyError(f'Key "{key}" not in the automation JSON!')

    def _post_automation_json(self, end_point, payload):
        url = self.API_MOBILE_AUTOMATION_URL.format(
            end_point=end_point, automation_id=self._automation_json["id"]
        )
        return self._nexia_home.post_url(url, payload)

    def update_automation_json(self, automation_json):
        """Update with new json from the api"""
        if self._automation_json is None:
            return

        _LOGGER.debug(
            "Updated automation_id:%s with new data from post",
            self.automation_id,
        )
        self._automation_json.update(automation_json)



"""Nexia Home."""

import datetime
import logging

import requests

from .automation import NexiaAutomation
from .const import (
    APP_VERSION,
    ASAIR_ROOT_URL,
    TRANE_ROOT_URL,
    BRAND_ASAIR,
    BRAND_NEXIA,
    BRAND_TRANE,
    DEFAULT_DEVICE_NAME,
    MOBILE_URL_TEMPLATE,
    NEXIA_ROOT_URL,
)
from .thermostat import NexiaThermostat
from .util import load_or_create_uuid

MAX_LOGIN_ATTEMPTS = 4
TIMEOUT = 20

_LOGGER = logging.getLogger(__name__)

DEVICES_ELEMENT = 0
AUTOMATIONS_ELEMENT = 1


class NexiaHome:
    """Nexia Home Access Class."""

    DEFAULT_UPDATE_RATE = 120  # 2 minutes
    DISABLE_AUTO_UPDATE = "Disable"

    def __init__(
        self,
        house_id=None,
        username=None,
        password=None,
        auto_login=True,
        auto_update=True,
        device_name=DEFAULT_DEVICE_NAME,
        brand=BRAND_NEXIA,
        state_file=None,
    ):
        """
        Connects to and provides the ability to get and set parameters of your
        Nexia connected thermostat.

        :param house_id: int - Your house_id. You can get this from logging in
        and looking at the url once you're looking at your climate device.
        https://www.mynexia.com/houses/<house_id>/climate
        :param username: str - Your login email address
        :param password: str - Your login password
        :param auto_login: bool - Default is True, Login now (True), or login
        manually later (False)
        :param auto_update: bool - Default is True, Update now (True), or update
        manually later (False)

        JSON update. Default is 300s.
        """

        self.username = username
        self.password = password
        self.house_id = house_id
        self.mobile_id = None
        self.brand = brand
        self.login_attempts_left = MAX_LOGIN_ATTEMPTS
        self._state_file = state_file or f"{brand}_config_{self.username}.conf"
        self.api_key = None
        self.devices_json = None
        self.automations_json = None
        self.last_update = None
        self._name = None
        self.thermostats = None
        self.automations = None
        self._device_name = device_name
        self._last_update_etag = None
        self._uuid = None

        # Create a session
        self.session = requests.session()
        self.session.max_redirects = 3

        # Login if requested
        if auto_login:
            self.login()

            if auto_update:
                self.update()

    @property
    def API_MOBILE_PHONE_URL(self):  # pylint: disable=invalid-name
        return f"{self.mobile_url}/phones"

    @property
    def API_MOBILE_SESSION_URL(self):  # pylint: disable=invalid-name
        return f"{self.mobile_url}/session"

    @property
    def API_MOBILE_HOUSES_URL(self):  # pylint: disable=invalid-name
        return self.mobile_url + "/houses/{house_id}"

    @property
    def API_MOBILE_ACCOUNTS_SIGN_IN_URL(self):  # pylint: disable=invalid-name
        return f"{self.mobile_url}/accounts/sign_in"

    @property
    def AUTH_FAILED_STRING(self):  # pylint: disable=invalid-name
        return f"{self.root_url}/login"

    @property
    def AUTH_FORGOTTEN_PASSWORD_STRING(self):  # pylint: disable=invalid-name
        return f"{self.root_url}/account/forgotten_credentials"

    @property
    def root_url(self):
        """The root url for the service."""
        if self.brand == BRAND_ASAIR:
            return ASAIR_ROOT_URL
        if self.brand == BRAND_TRANE:
            return TRANE_ROOT_URL
        return NEXIA_ROOT_URL

    @property
    def mobile_url(self):
        """The mobile url for the service."""
        return MOBILE_URL_TEMPLATE.format(self.root_url)

    def _api_key_headers(self):
        return {
            "X-AppVersion": APP_VERSION,
            "X-MobileId": str(self.mobile_id),
            "X-ApiKey": str(self.api_key),
        }

    def post_url(self, request_url: str, payload: dict):
        """
        Posts data to the session from the url and payload
        :param url: str
        :param payload: dict
        :return: response
        """
        _LOGGER.debug("POST: Calling url %s with payload: %s", request_url, payload)

        response = self.session.post(
            request_url, payload, timeout=TIMEOUT, headers=self._api_key_headers()
        )

        _LOGGER.debug("POST: Response from url %s: %s", request_url, response.content)
        if response.status_code == 302:
            # assuming its redirecting to login
            _LOGGER.debug(
                "POST Response returned code 302, re-attempting login and resending request."
            )
            self.login()
            return self.post_url(request_url, payload)

        # no need to sleep anymore as we consume the response and update the thermostat's JSON

        response.raise_for_status()
        return response

    def _get_url(self, request_url, headers=None):
        """
        Returns the full session.get from the URL (ROOT_URL + url)
        :param url: str
        :return: response
        """
        if not headers:
            headers = {}

        headers.update(self._api_key_headers())
        _LOGGER.debug("GET: Calling url %s", request_url)
        response = self.session.get(
            request_url,
            allow_redirects=False,
            timeout=TIMEOUT,
            headers=headers,
        )
        _LOGGER.debug(
            "GET: RESPONSE %s: response.status_code %s",
            request_url,
            response.status_code,
        )

        if response.status_code == 302:
            _LOGGER.debug(
                "GET Response returned code 302, re-attempting login and resending request."
            )
            # assuming its redirecting to login
            self.login()
            return self._get_url(request_url)

        response.raise_for_status()
        return response

    @staticmethod
    def _check_response(error_text, request):
        """
        Checks the request response, throws exception with the description text
        :param error_text: str
        :param request: response
        :return: None
        """
        if request is None or request.status_code != 200:
            if request is not None:
                response = ""
                for key in request.__attrs__:
                    response += f"  {key}: {getattr(request, key)}\n"
                raise Exception(f"{error_text}\n{response}")
            raise Exception(f"No response from session. {error_text}")

    def _find_house_id(self):
        """Finds the house id if none is provided."""
        request = self.post_url(
            self.API_MOBILE_SESSION_URL,
            {"app_version": APP_VERSION, "device_uuid": str(self._uuid)},
        )
        if request and request.status_code == 200:
            ts_json = request.json()
            if ts_json:
                data = ts_json["result"]["_links"]["child"][0]["data"]
                self.house_id = data["id"]
                self._name = data["name"]
            else:
                raise Exception("Nothing in the JSON")
        else:
            self._check_response(
                "Failed to get house id JSON, session probably timed" " out",
                request,
            )

    def update_from_json(self, json_dict: dict):
        """Update the json from the houses endpoint if fetched externally."""
        self._name = json_dict["result"]["name"]
        self.devices_json = _extract_devices_from_houses_json(json_dict)
        self.automations_json = _extract_automations_from_houses_json(json_dict)
        self._update_devices()
        self._update_automations()

    def update(self, force_update=True):
        """
        Forces a status update from nexia
        :return: None
        """
        if not self.mobile_id:
            # not yet authenticated
            return

        headers = {}
        if self._last_update_etag:
            headers["If-None-Match"] = self._last_update_etag

        response = self._get_url(
            self.API_MOBILE_HOUSES_URL.format(house_id=self.house_id), headers=headers
        )

        if not response:
            self._check_response(
                "Failed to get house JSON, session probably timed out",
                response,
            )
            return
        if response.status_code == 304:
            _LOGGER.debug("Update returned 304")
            # already up to date
            return
        if response.status_code != 200:
            self._check_response(
                "Unexpected http status while fetching house JSON",
                response,
            )
            return

        ts_json = response.json()
        if ts_json:
            self._name = ts_json["result"]["name"]
            self.devices_json = _extract_devices_from_houses_json(ts_json)
            self.automations_json = _extract_automations_from_houses_json(ts_json)
            self._last_update_etag = response.headers.get("etag")
        else:
            raise Exception("Nothing in the JSON")
        self._update_devices()
        self._update_automations()
        return

    def _update_devices(self):
        self.last_update = datetime.datetime.now()

        if self.thermostats is None:
            self.thermostats = []
            for thermostat_json in self.devices_json:
                if (
                    "type" in thermostat_json
                    and "thermostat" not in thermostat_json["type"]
                ):
                    # Not a thermostat
                    continue
                nexia_thermostat = NexiaThermostat(self, thermostat_json)
                if not nexia_thermostat.get_zone_ids():
                    # No zones (likely an xl624 which is not supported at this time)
                    continue
                self.thermostats.append(nexia_thermostat)
            return

        thermostat_updates_by_id = {}
        for thermostat_json in self.devices_json:
            if (
                "type" in thermostat_json
                and "thermostat" not in thermostat_json["type"]
            ):
                # Not a thermostat
                continue
            thermostat_updates_by_id[thermostat_json["id"]] = thermostat_json

        for thermostat in self.thermostats:
            if thermostat.thermostat_id in thermostat_updates_by_id:
                thermostat.update_thermostat_json(
                    thermostat_updates_by_id[thermostat.thermostat_id]
                )

    def _update_automations(self):
        self.last_update = datetime.datetime.now()

        if self.automations is None:
            self.automations = []
            for automation_json in self.automations_json:
                self.automations.append(NexiaAutomation(self, automation_json))
            return

        automation_updates_by_id = {}
        for automation_json in self.automations_json:
            automation_updates_by_id[automation_json["id"]] = automation_json

        for automation in self.automations:
            if automation.automation_id in automation_updates_by_id:
                automation.update_automation_json(
                    automation_updates_by_id[automation.automation_id]
                )

    ########################################################################
    # Session Methods

    def login(self):
        """
        Provides you with a Nexia web session.

        All parameters should be set prior to calling this.
        - username - (str) Your email address
        - password - (str) Your login password
        - house_id - (int) Your house id
        :return: None
        """
        self._uuid = load_or_create_uuid(self._state_file)
        if self.login_attempts_left > 0:
            payload = {
                "login": self.username,
                "password": self.password,
                "children": [],
                "childSchemas": [],
                "commitModel": None,
                "nextHref": None,
                "device_uuid": str(self._uuid),
                "device_name": self._device_name,
                "app_version": APP_VERSION,
                "is_commercial": False,
            }
            request = self.post_url(self.API_MOBILE_ACCOUNTS_SIGN_IN_URL, payload)

            if (
                request is None
                or request.status_code != 200
                and request.status_code != 302
            ):
                self.login_attempts_left -= 1
            self._check_response("Failed to login", request)

            if request.url == self.AUTH_FORGOTTEN_PASSWORD_STRING:
                raise Exception(
                    f"Failed to login, getting redirected to {request.url}"
                    f". Try to login manually on the website."
                )

            json_dict = request.json()
            if json_dict.get("success") is not True:
                error_text = json_dict.get("error", "Unknown Error")
                raise Exception(f"Failed to login, {error_text}")

            self.mobile_id = json_dict["result"]["mobile_id"]
            self.api_key = json_dict["result"]["api_key"]
        else:
            raise Exception(
                f"Failed to login after {MAX_LOGIN_ATTEMPTS} attempts! Any "
                f"more attempts may lock your account!"
            )

        if not self.house_id:
            self._find_house_id()

    def get_name(self):
        """Name of the house"""
        return self._name

    def get_last_update(self):
        """
        Returns a string indicating the ISO formatted time string of the last
        update
        :return: The ISO formatted time string of the last update,
        datetime.datetime.min if never updated
        """
        if self.last_update is None:
            return datetime.datetime.isoformat(datetime.datetime.min)
        return datetime.datetime.isoformat(self.last_update)

    def get_thermostat_by_id(self, thermostat_id):
        """Get a thermostat by its nexia id."""
        for thermostat in self.thermostats:
            if thermostat.thermostat_id == thermostat_id:
                return thermostat
        raise KeyError

    def get_thermostat_ids(self):
        """
        Returns the number of thermostats available to Nexia
        :return:
        """
        ids = list()
        for thermostat in self.thermostats:
            ids.append(thermostat.thermostat_id)
        return ids

    def get_automation_by_id(self, automation_id):
        """Get a automation by its nexia id."""
        for automation in self.automations:
            if automation.automation_id == automation_id:
                return automation
        raise KeyError

    def get_phone_ids(self):
        """Get all the mobile phone ids."""
        response = self._get_url(self.API_MOBILE_PHONE_URL)
        data = response.json()
        items = data["result"]["items"]
        phones = []
        for phone in items:
            phones.append(phone["phone_id"])
        return phones

    def get_automation_ids(self):
        """
        Returns the number of automations available to Nexia
        :return:
        """
        ids = list()
        for automation in self.automations:
            ids.append(automation.automation_id)
        return ids


def _extract_devices_from_houses_json(json_dict: dict):
    """Extras the payload from the houses json endpoint data."""
    return _extract_items(
        json_dict["result"]["_links"]["child"][DEVICES_ELEMENT]["data"]
    )


def _extract_automations_from_houses_json(json_dict: dict):
    """Extras the payload from the houses json endpoint data."""
    return _extract_items(
        json_dict["result"]["_links"]["child"][AUTOMATIONS_ELEMENT]["data"]
    )


def _extract_items(json_dict: dict):
    """Return the items key if it exists, otherwise the top level."""
    return json_dict.get("items", json_dict)



#!/usr/bin/python3

"""Nexia Climate Device Access"""

import argparse

from nexia.home import NexiaHome
from nexia.const import BRAND_NEXIA

parser = argparse.ArgumentParser()
parser.add_argument("--brand", type=str, help="Brand (nexia or asair).")
parser.add_argument("--username", type=str, help="Your username/email address.")
parser.add_argument("--password", type=str, help="Your password.")
args = parser.parse_args()
brand = args.brand or BRAND_NEXIA

if args.username and args.password:
    nexia_home = NexiaHome(username=args.username, password=args.password, brand=brand)
else:
    parser.print_help()
    exit()

print(nexia_home.devices_json)
print(nexia_home.automations_json)



#!/usr/bin/python3

"""Nexia Climate Device Access"""

import argparse
import pprint

from nexia.home import NexiaHome

parser = argparse.ArgumentParser()
parser.add_argument("--username", type=str, help="Your Nexia username/email address.")
parser.add_argument("--password", type=str, help="Your Nexia password.")
args = parser.parse_args()

if args.username and args.password:
    nexia_home = NexiaHome(username=args.username, password=args.password)
else:
    parser.print_help()
    exit()

count = 0
for phone_id in nexia_home.get_phone_ids():
    count += 1
    if count < 5:
        continue

    print("Delete phone id: ", phone_id)
    response = nexia_home.session.delete(
        "https://www.mynexia.com/mobile/phones/" + str(phone_id),
        headers=nexia_home._api_key_headers(),
    )
    pprint.pprint(response)



"""Nexia Themostat Zone."""

import logging
import math

from .const import (
    DAMPER_CLOSED,
    HOLD_PERMANENT,
    OPERATION_MODE_COOL,
    OPERATION_MODE_HEAT,
    OPERATION_MODES,
    PRESET_MODE_NONE,
    UNIT_CELSIUS,
    ZONE_IDLE,
)
from .util import find_dict_with_keyvalue_in_json

_LOGGER = logging.getLogger(__name__)


class NexiaThermostatZone:
    """A nexia thermostat zone."""

    def __init__(self, nexia_home, nexia_thermostat, zone_json):
        """Create a nexia zone."""
        self._nexia_home = nexia_home
        self._zone_json = zone_json
        self.thermostat = nexia_thermostat
        self.zone_id = zone_json["id"]

    @property
    def API_MOBILE_ZONE_URL(self):  # pylint: disable=invalid-name
        return self._nexia_home.mobile_url + "/xxl_zones/{zone_id}/{end_point}"

    def get_name(self):
        """
        Returns the zone name
        :return: str
        """
        name = str(self._get_zone_key("name"))
        if name == "NativeZone":
            name = f"{self.thermostat.get_name()} NativeZone"
        return name

    def get_cooling_setpoint(self):
        """
        Returns the cooling setpoint in the temperature unit of the thermostat
        :return: int
        """
        return self._get_zone_key("setpoints")["cool"]

    def get_heating_setpoint(self):
        """
        Returns the heating setpoint in the temperature unit of the thermostat
        :return: int
        """
        return self._get_zone_key("setpoints")["heat"]

    def get_current_mode(self):
        """
        Returns the current mode of the zone. This may not match the requested
        mode

        :return: str
        """
        return self._get_zone_setting("zone_mode")["current_value"].upper()

    def get_requested_mode(self):
        """
        Returns the requested mode of the zone. This should match the zone's
        mode on the thermostat.
        Available options can be found in NexiaThermostat.OPERATION_MODES
        :return: str
        """
        return self._get_zone_features("thermostat_mode")["value"].upper()

    def get_temperature(self):
        """
        Returns the temperature of the zone in the temperature unit of the
        thermostat.
        :return: int
        """
        return self._get_zone_key("temperature")

    def get_presets(self):
        """
        Supposed to return the zone presets. For some reason, most of the time,
        my unit only returns "AWAY", but I can set the other modes. There is
        the capability to add additional zone presets on the main thermostat,
        so this may not work as expected.

        :return:
        """
        options = self._get_zone_setting("preset_selected")["options"]
        return [opt["label"] for opt in options]

    def get_preset(self):
        """
        Returns the zone's currently selected preset. Should be one of the
        strings in NexiaThermostat.get_zone_presets().
        :return: str
        """
        preset_selected = self._get_zone_setting("preset_selected")
        return preset_selected["labels"][preset_selected["current_value"]]

    def get_status(self):
        """
        Returns the zone status.
        :return: str
        """
        return self._get_zone_key("zone_status") or ZONE_IDLE

    def _get_zone_run_mode(self):
        """
        Returns the run mode ("permanent_hold", "run_schedule")
        :return: str
        """
        # Will be None is scheduling is disabled
        return self._get_zone_setting_or_none("run_mode")

    def get_setpoint_status(self):
        """
        Returns the setpoint status, like "Following Schedule - Home", or
        "Permanent Hold"
        :return: str
        """
        run_mode = self._get_zone_run_mode()
        if not run_mode:
            # Scheduling is disabled
            return "Permanent Hold"

        run_mode_current_value = run_mode["current_value"]
        run_mode_label = find_dict_with_keyvalue_in_json(
            run_mode["options"], "value", run_mode_current_value
        )["label"]

        if run_mode_current_value == HOLD_PERMANENT:
            return run_mode_label

        preset_label = self.get_preset()
        if run_mode_current_value == PRESET_MODE_NONE:
            return run_mode_label
        return f"{run_mode_label} - {preset_label}"

    def is_calling(self):
        """
        Returns True if the zone is calling for heat/cool.
        :return: bool
        """
        operating_state = self._get_zone_key("operating_state")
        if not operating_state or operating_state == DAMPER_CLOSED:
            return False
        return True

    def check_heat_cool_setpoints(self, heat_temperature=None, cool_temperature=None):
        """
        Checks the heat and cool setpoints to check if they are within the
        appropriate range and within the deadband limits.

        Will throw exception if not valid.
        :param heat_temperature: int
        :param cool_temperature: int
        :return: None
        """

        deadband = self.thermostat.get_deadband()
        (
            min_temperature,
            max_temperature,
        ) = self.thermostat.get_setpoint_limits()

        if heat_temperature is not None:
            heat_temperature = self.round_temp(heat_temperature)
        if cool_temperature is not None:
            cool_temperature = self.round_temp(cool_temperature)

        if (
            heat_temperature is not None
            and cool_temperature is not None
            and not heat_temperature < cool_temperature
        ):
            raise AttributeError(
                f"The heat setpoint ({heat_temperature}) must be less than the"
                f" cool setpoint ({cool_temperature})."
            )
        if (
            heat_temperature is not None
            and cool_temperature is not None
            and not cool_temperature - heat_temperature >= deadband
        ):
            raise AttributeError(
                f"The heat and cool setpoints must be at least {deadband} "
                f"degrees different."
            )
        if heat_temperature is not None and not heat_temperature <= max_temperature:
            raise AttributeError(
                f"The heat setpoint ({heat_temperature} must be less than the "
                f"maximum temperature of {max_temperature} degrees."
            )
        if cool_temperature is not None and not cool_temperature >= min_temperature:
            raise AttributeError(
                f"The cool setpoint ({cool_temperature}) must be greater than "
                f"the minimum temperature of {min_temperature} degrees."
            )
        # The heat and cool setpoints appear to be valid.

    def is_in_permanent_hold(self):
        """
        Returns True if the zone is in a permanent hold.
        :return: bool
        """
        run_mode = self._get_zone_run_mode()
        if not run_mode:
            # Scheduling is disabled
            return True
        return run_mode["current_value"] == HOLD_PERMANENT

    ########################################################################
    # Zone Set Methods

    def call_return_to_schedule(self):
        """
        Tells the zone to return to its schedule.
        :return: None
        """

        # Set the thermostat
        self._post_and_update_zone_json("return_to_schedule", {})

    def call_permanent_hold(
        self,
        heat_temperature=None,
        cool_temperature=None,
    ):
        """
        Tells the zone to call a permanent hold. Optionally can provide the
        temperatures.
        :param heat_temperature:
        :param cool_temperature:
        :return:
        """

        if heat_temperature is None and cool_temperature is None:
            # Just calling permanent hold on the current temperature
            heat_temperature = self.get_heating_setpoint()
            cool_temperature = self.get_cooling_setpoint()
        elif heat_temperature is not None and cool_temperature is not None:
            # Both heat and cool setpoints provided, continue
            pass
        else:
            # Not sure how I want to handle only one temperature provided, but
            # this definitely assumes you're using auto mode.
            raise AttributeError(
                "Must either provide both heat and cool setpoints, or don't "
                "provide either"
            )

        self._set_hold_and_setpoints(
            cool_temperature,
            heat_temperature,
        )

    def set_heat_cool_temp(
        self,
        heat_temperature=None,
        cool_temperature=None,
        set_temperature=None,
    ):
        """
        Sets the heat and cool temperatures of the zone. You must provide
        either heat and cool temperatures, or just the set_temperature. This
        method will add deadband to the heat and cool temperature from the set
        temperature.

        :param heat_temperature: int or None
        :param cool_temperature: int or None
        :param set_temperature: int or None
        :return: None
        """
        deadband = self.thermostat.get_deadband()

        if set_temperature is None or (heat_temperature and cool_temperature):
            if heat_temperature:
                heat_temperature = self.round_temp(heat_temperature)
            else:
                heat_temperature = min(
                    self.get_heating_setpoint(),
                    self.round_temp(cool_temperature) - deadband,
                )

            if cool_temperature:
                cool_temperature = self.round_temp(cool_temperature)
            else:
                cool_temperature = max(
                    self.get_cooling_setpoint(),
                    self.round_temp(heat_temperature) + deadband,
                )

        else:
            # This will smartly select either the ceiling of the floor temp
            # depending on the current operating mode.
            zone_mode = self.get_current_mode()
            if zone_mode == OPERATION_MODE_COOL:
                cool_temperature = self.round_temp(set_temperature)
                heat_temperature = min(
                    self.get_heating_setpoint(),
                    self.round_temp(cool_temperature) - deadband,
                )
            elif zone_mode == OPERATION_MODE_HEAT:
                heat_temperature = self.round_temp(set_temperature)
                cool_temperature = max(
                    self.get_cooling_setpoint(),
                    self.round_temp(heat_temperature) + deadband,
                )
            else:
                cool_temperature = self.round_temp(set_temperature) + math.ceil(
                    deadband / 2
                )
                heat_temperature = self.round_temp(set_temperature) - math.ceil(
                    deadband / 2
                )

        self._set_setpoints(cool_temperature, heat_temperature)

    def _set_hold_and_setpoints(self, cool_temperature, heat_temperature):
        # Set the thermostat
        run_mode = self._get_zone_run_mode()
        if run_mode:
            if run_mode["current_value"] != HOLD_PERMANENT:
                self._post_and_update_zone_json("run_mode", {"value": HOLD_PERMANENT})

        self._set_setpoints(cool_temperature, heat_temperature)

    def _set_setpoints(self, cool_temperature, heat_temperature):
        # Check that the setpoints are valid
        self.check_heat_cool_setpoints(heat_temperature, cool_temperature)
        zone_cooling_setpoint = self.get_cooling_setpoint()
        zone_heating_setpoint = self.get_heating_setpoint()
        if (
            zone_cooling_setpoint != cool_temperature
            or heat_temperature != zone_heating_setpoint
        ):
            self._post_and_update_zone_json(
                "setpoints", {"heat": heat_temperature, "cool": cool_temperature}
            )

    def set_preset(self, preset):
        """
        Sets the preset of the specified zone.
        :param preset: str - The preset, see
        NexiaThermostat.get_zone_presets(zone_id)
        :return: None
        """
        if self.get_preset() != preset:
            preset_selected = self._get_zone_setting("preset_selected")
            value = 0
            for option in preset_selected["options"]:
                if option["label"] == preset:
                    value = option["value"]
                    break
            self._post_and_update_zone_json("preset_selected", {"value": value})

    def set_mode(self, mode):
        """
        Sets the mode of the zone.
        :param mode: str - The mode, see NexiaThermostat.OPERATION_MODES
        :return:
        """
        # Validate the data
        if mode in OPERATION_MODES:
            self._post_and_update_zone_json("zone_mode", {"value": mode})
        else:
            raise KeyError(
                f'Invalid mode "{mode}". Select one of the following: '
                f"{OPERATION_MODES}"
            )

    def round_temp(self, temperature: float):
        """
        Rounds the temperature to the nearest 1/2 degree for C and neareast 1
        degree for F
        :param temperature: temperature to round
        :return: float rounded temperature
        """
        if self.thermostat.get_unit() == UNIT_CELSIUS:
            temperature *= 2
            temperature = round(temperature)
            temperature /= 2
        else:
            temperature = round(temperature)
        return temperature

    @property
    def _has_zoning(self):
        """Returns if zoning is enabled."""
        return bool(self._zone_json["settings"])

    def _get_zone_setting(self, key):
        """
        Returns the zone value for the key and zone_id provided.
        :param key: str
        :return: The value of the key/value pair.
        """

        if not self._has_zoning:
            if key == "zone_mode":
                key = "system_mode"
            return self.thermostat.get_thermostat_settings_key(key)

        zone = self._zone_json
        subdict = find_dict_with_keyvalue_in_json(zone["settings"], "type", key)
        if not subdict:
            raise KeyError(f'Zone settings key "{key}" invalid.')
        return subdict

    def _get_zone_setting_or_none(self, key):
        """
        Returns the zone value from the provided key in the zones's
        JSON.
        :param key: str
        :return: value
        """
        try:
            return self._get_zone_setting(key)
        except KeyError:
            return None

    def _get_zone_features(self, key):
        """
        Returns the zone value for the key provided.
        :param key: str

        :return: The value of the key/value pair.
        """
        zone = self._zone_json
        subdict = find_dict_with_keyvalue_in_json(zone["features"], "name", key)
        if not subdict:
            raise KeyError(f'Zone feature key "{key}" invalid.')
        return subdict

    def _get_zone_key(self, key):
        """
        Returns the zone value for the key provided.
        :param key: str
        :return: The value of the key/value pair.
        """
        if key in self._zone_json:
            return self._zone_json[key]

        raise KeyError(f'Zone key "{key}" invalid.')

    def _post_and_update_zone_json(self, end_point, payload):
        url = self.API_MOBILE_ZONE_URL.format(end_point=end_point, zone_id=self.zone_id)
        response = self._nexia_home.post_url(url, payload)
        self.update_zone_json(response.json()["result"])

    def update_zone_json(self, zone_json):
        """Update with new json from the api"""
        if self._zone_json is None:
            return

        _LOGGER.debug(
            "Updated thermostat_id:%s zone_id:%s with new data from post",
            self.thermostat.thermostat_id,
            self.zone_id,
        )
        self._zone_json.update(zone_json)
